@@ -20,18 +20,13 @@ package coremain
 import (
 	"bytes"
 	"github.com/IrineSistiana/mosdns/v3/dispatcher/handler"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/mlog"
 	"gopkg.in/yaml.v3"
 	"os"
 )
 
-// Config is config
 type Config struct {
-	Log struct {
-		Level    string `yaml:"level"`
-		File     string `yaml:"file"`
-		ErrFile  string `yaml:"err_file"`
-		InfoFile string `yaml:"info_file"`
-	} `yaml:"log"`
+	Log     mlog.LogConfig    `yaml:"log"`
 	Library []string          `yaml:"library"`
 	Plugin  []*handler.Config `yaml:"plugin"`
 	Include []string          `yaml:"include"`
